@@ -65,7 +65,7 @@ u_char param_select[] = {
    LABEL | OBJINST,				/* P_ROTATION */
    LABEL | OBJINST,				/* P_SCALE */
    POLYGON | SPLINE | ARC | PATH,		/* P_LINEWIDTH */
-   ALL_TYPES, 					/* P_COLOR (tbd) */
+   ALL_TYPES, 					/* P_COLOR */
    ALL_TYPES, 					/* P_EXPRESSION */
    POLYGON | SPLINE | LABEL | OBJINST | ARC 	/* P_POSITION */
 };
@@ -198,14 +198,21 @@ oparamptr match_instance_param(objinstptr thisinst, char *key)
 /*----------------------------------------------------------------------*/
 /* Find the parameter in the indicated instance by key.  If no such 	*/
 /* instance value exists, return the object (default) parameter.	*/
+/*									*/
+/* find_param() hides instances of expression parameters, returning the	*/
+/* default parameter value.  In cases where the instance value (last	*/
+/* evaluated expression result) is needed, use match_instance_param().	*/
+/* An exception is made when the instance param has type XC_EXPR,	*/
+/* indicating that the instance redefines the entire expression.	*/
 /*----------------------------------------------------------------------*/
 
 oparamptr find_param(objinstptr thisinst, char *key)
 {
-   oparamptr fparam;
+   oparamptr fparam, ops;
    fparam = match_instance_param(thisinst, key);
-   if (fparam == NULL)
-      fparam = match_param(thisinst->thisobject, key);
+   ops = match_param(thisinst->thisobject, key);
+   if ((fparam == NULL) || (ops->type == XC_EXPR) && (fparam->type != XC_EXPR))
+      fparam = ops;
    return fparam;
 }
 
@@ -278,6 +285,21 @@ void free_instance_param(objinstptr thisinst, oparamptr thisparam)
    }
 }
 
+/*----------------------------------------------------------------------*/
+/* Convenience function used by files.c to set a color parameter.	*/
+/*----------------------------------------------------------------------*/
+
+void std_eparam(genericptr gen, char *key)
+{
+   eparamptr epp;
+
+   if (key == NULL) return;
+
+   epp = make_new_eparam(key);
+   epp->next = gen->passed;
+   gen->passed = epp;
+}
+
 /*----------------------------------------------*/
 /* Draw a circle at all parameter positions	*/
 /*----------------------------------------------*/
@@ -414,7 +436,7 @@ int opsubstitute(objectptr thisobj, objinstptr pinst)
    int i, j, k, ival;
    char *key;
    float fval;
-   oparamptr dps, ops;
+   oparamptr dps, ops, cps;
    eparamptr epp;
    XPoint *setpt;
    Boolean needrecalc;	/* for arcs and splines */
@@ -458,7 +480,7 @@ int opsubstitute(objectptr thisobj, objinstptr pinst)
 		  free(promoted);
 		  break;
 	       case XC_EXPR:
-		  if ((promoted = evaluate_expr(dps)) == NULL) continue;
+		  if ((promoted = evaluate_expr(dps, pinst)) == NULL) continue;
 		  if (sscanf(promoted, "%g", &fval) == 1)
 		     ival = (int)(fval + 0.5);
 		  free(promoted);
@@ -651,6 +673,10 @@ int opsubstitute(objectptr thisobj, objinstptr pinst)
 		     break;
 	       }
 	       break;
+	    case P_COLOR:
+	       retval = max(retval, 0);
+	       thiselem->color = ival;
+	       break;
          }
       }
       /* substitutions into arcs and splines require that the	*/
@@ -885,6 +911,9 @@ void pwriteback(objinstptr thisinst)
 		           break;
 	             }
 	             break;
+		  case P_COLOR:
+		     wtemp.ival = thiselem->color;
+		     break;
                }
 	       destivalptr = (ips != NULL) ? &ips->parameter.ivalue
 			: &ops->parameter.ivalue;
@@ -1015,9 +1044,13 @@ void resolveparams(objinstptr thisinst)
 	    }
 	    break;
 	 case XC_EXPR:
-	    if (!strcmp(ops->parameter.expr, ips->parameter.expr)) {
-	       free(ips->parameter.expr);
-	       free_instance_param(thisinst, ips);
+	    /* Always retain the last evaluated result, unless	*/
+	    /* the instance redefines the expression.		*/
+	    if (ips->type == XC_EXPR) {
+	       if (!strcmp(ops->parameter.expr, ips->parameter.expr)) {
+	          free(ips->parameter.expr);
+	          free_instance_param(thisinst, ips);
+	       }
 	    }
 	    break;
 	 case XC_INT: case XC_FLOAT:
@@ -1099,11 +1132,17 @@ oparamptr copyparameter(oparamptr cops)
 /* Fill any NULL instance parameters with the values	*/
 /* from the calling instance, or from the instance	*/
 /* object's defaults if destinst = sourceinst.		*/ 
+/*							*/
+/* Expression parameters get special treatment because	*/
+/* the instance value may be holding the last evaluated	*/
+/* expression, not an instance value of the expression.	*/
+/* If so, its type will be XC_STRING or XC_FLOAT, not	*/
+/* XC_EXPR.						*/
 /*------------------------------------------------------*/
 
 void copyparams(objinstptr destinst, objinstptr sourceinst)
 {
-   oparamptr psource, cops, newops;
+   oparamptr psource, cops, newops, ips;
 
    if (sourceinst == NULL) return;
    if (destinst == sourceinst)
@@ -1112,7 +1151,8 @@ void copyparams(objinstptr destinst, objinstptr sourceinst)
       psource = sourceinst->params;
 
    for (cops = psource; cops != NULL; cops = cops->next) {
-      if (match_instance_param(destinst, cops->key) == NULL) {
+      if (((ips = match_instance_param(destinst, cops->key)) == NULL) ||
+		(cops->type == XC_EXPR && ips->type != XC_EXPR)) {
          newops = copyparameter(cops);
 	 newops->next = destinst->params;
 	 destinst->params = newops;
@@ -1320,6 +1360,9 @@ void makenumericalp(genericptr *gelem, u_int mode, char *key)
    }
 
    if (newops) {
+      if (mode == P_COLOR)
+	 newops->parameter.ivalue = (int)((*gelem)->color);
+
       switch((*gelem)->type) {
          case LABEL:
 	    switch(mode) {
@@ -1648,6 +1691,7 @@ void makeparam(labelptr thislabel, char *key)
    begpart->data.string = (char *)malloc(1 + strlen(newkey));
    strcpy(begpart->data.string, newkey);
    endpart->type = PARAM_END;
+   endpart->data.string = (u_char *)NULL;
 
    /* Now move the sections of string to the object parameter */
 
