@@ -715,11 +715,11 @@ extern void selectfilter(xcWidget, pointertype, caddr_t);
 extern Boolean checkselect(short);
 extern void geneasydraw(short, short, objectptr, objinstptr);
 extern void gendrawselected(short *, objectptr, objinstptr);
-extern short *genselectelement(short, u_char, objectptr, objinstptr);
+extern selection *genselectelement(short, u_char, objectptr, objinstptr);
 extern short *allocselect();
 extern void setoptionmenu();
 extern int test_insideness(int, int, XPoint *);
-extern Boolean pathselect(genericptr *, short);
+extern Boolean pathselect(genericptr *, short, float);
 extern Boolean areaelement(genericptr *);
 extern void selectarea();
 extern void startdesel(xcWidget, caddr_t, caddr_t);
@@ -729,7 +729,8 @@ extern void draw_all_selected();
 extern void clearselects_noundo();
 extern void clearselects();
 extern void unselect_all();
-extern genericptr recurselect(short, pushlistptr *);
+extern selection *recurselect(short, u_char, pushlistptr *);
+extern short *recurse_select_element(short, u_char);
 #ifdef TCL_WRAPPER
 extern xcTimeOutProc startselect(caddr_t);
 #else
